#!/home/artur/programs/tcl8.4.3-atkdebugger/unix/tclsh
package require Tk

if {[file exists libatkdebugger0.21.so]} {
  load libatkdebugger0.21.so
} else {
  package require atkdebugger
}

namespace eval Debugger {
    variable mainWindow {}
    variable debugSync {}
    variable breakpointsWindow {}
    variable selectedLevel
    variable notHideDebugger 0
}
proc Debugger::atkDebugger {} {
    variable mainWindow 
    variable debugSync
    variable debugMode 1
    set dlevel [atk::debuglevel]
    InitGUIThenNeeded
    # We make own execution loop. We do not want to lose debugging context.

    for {set x $dlevel} {$x>0} {incr x -1} {
	$mainWindow.levelf.listbox insert end [lindex [info level $x] 0]
    }
    $mainWindow.levelf.listbox insert end "global level"
    if {[atk::wasError]} {
	global errorInfo;
	$mainWindow.vartext delete 1.0 end
	$mainWindow.vartext insert 1.0 $errorInfo
	wm title $mainWindow {Error was occured : ATK Base Debugger}
    } else {
	wm title $mainWindow {ATK Base Debugger}
    }
    showDebugPlace $dlevel
    # We make own execution loop. We do not want to lose debugging context.
    while 1 {
	update idletasks
	vwait Debugger::debugSync
	eval $debugSync
    }
}
proc Debugger::showDebugPlace {level} {
    variable selectedLevel
    set selectedLevel $level
    set res [atk::runplace $level]
    variable mainWindow
    $mainWindow.procTextFrame.text delete 1.0 end
    $mainWindow.procTextFrame.text insert 1.0 [lindex $res 0]
    set cur [lindex $res 1]
    $mainWindow.procTextFrame.text tag add hlight "1.0 + $cur char" "1.0 + [expr {$cur + [lindex $res 2]}] char"
    $mainWindow.procTextFrame.text see "1.0 + $cur char"
    $mainWindow.varsf.listbox delete 0 end
    foreach var [uplevel #$level {::info vars}] {
	$mainWindow.varsf.listbox insert end $var	 
    }
}
proc Debugger::InitGUIThenNeeded {} {
    variable mainWindow
    if {$mainWindow!=""} {
	$mainWindow.levelf.listbox delete 0 end
	$mainWindow.varsf.listbox delete 0 end
	$mainWindow.buttons.resume configure -state normal
	$mainWindow.buttons.stepInto configure -state normal
	$mainWindow.buttons.stepOver configure -state normal
	wm deiconify $mainWindow
	return
    }
    set mainWindow [toplevel .debugFrame]

    menu $mainWindow.mb -tearoff 0
    menu $mainWindow.mb.file -tearoff 0

    wm title $mainWindow {ATK Base Debugger}

    $mainWindow.mb add cascade -label Debugger -menu $mainWindow.mb.file  -underline 0
    $mainWindow.mb.file add command -label "Debug File" -command Debugger::debuggFile
    $mainWindow.mb.file add command -label "Breakpoints" -command Debugger::startBreakpointsBrowser
    global tcl_platform
    if {$tcl_platform(platform)=="windows"} {
	$mainWindow.mb.file add command -label "Console" -command [list console show]
    }
    global tcl_platform
    if {$tcl_platform(os)=="Windows"} {
	button $breakpointsWindow.add -text "Console" -command [list console show]
    }
    $mainWindow.mb.file add command -label Info -command Debugger::displayInfo
    $mainWindow configure -menu $mainWindow.mb

    set win [frame $mainWindow.procTextFrame]
    scrollbar $win.scroll -command [list $win.text yview] -takefocus 0
    text $win.text -height 15 -yscroll [list $win.scroll set]
    pack $win.scroll -side right -fill y
    pack $win.text -expand yes -fill both    

    text $mainWindow.vartext -width 50 -height 8

    set win [frame $mainWindow.levelf]
    scrollbar $win.scroll -command [list $win.listbox yview] -takefocus 0
    listbox $win.listbox -yscroll [list $win.scroll set] -exportselection no -height 5
    pack $win.scroll -side right -fill y
    pack $win.listbox -expand yes -fill both    
    bind $win.listbox <<ListboxSelect>> Debugger::selectLevel

    set win [frame $mainWindow.varsf]
    scrollbar $win.scroll -command [list $win.listbox yview] -takefocus 0
    listbox $win.listbox -yscroll [list $win.scroll set]  -exportselection no -height 8
    pack $win.scroll -side right -fill y
    pack $win.listbox -expand yes -fill both    
    bind $win.listbox <<ListboxSelect>> Debugger::selectVar

    $mainWindow.procTextFrame.text tag configure hlight -background yellow
    frame $mainWindow.buttons
    button $mainWindow.buttons.resume -text Resume -command Debugger::invokeResume
    button $mainWindow.buttons.stepInto -text {Step Into} -command Debugger::invokeStepInto
    button $mainWindow.buttons.stepOver -text {Step Over} -command Debugger::invokeStepOver
    pack $mainWindow.buttons.stepInto $mainWindow.buttons.stepOver $mainWindow.buttons.resume -side left
    grid $mainWindow.levelf -column 0 -row 0 -columnspan 2 -sticky news
    grid $mainWindow.varsf -column 0 -row 1 -sticky news
    grid $mainWindow.procTextFrame -column 0 -row 3 -columnspan 2 -sticky news
    grid $mainWindow.buttons  -column 0 -row 2 -columnspan 2 -sticky w
    grid $mainWindow.vartext  -column 1 -row 1 -sticky news
    grid rowconfigure $mainWindow 3 -weight 1
    grid rowconfigure $mainWindow 0 -weight 1
    grid columnconfigure $mainWindow 0 -weight 1
    wm protocol $mainWindow WM_DELETE_WINDOW Debugger::invokeResume
}
proc Debugger::selectLevel {} {
    variable mainWindow
    variable debugSync
    variable debugMode
    if {$debugMode} {
	set debugSync "Debugger::showDebugPlace [expr {[atk::debuglevel] - [$mainWindow.levelf.listbox curselection]}]"
    }
}
proc Debugger::selectVar {} {
    variable mainWindow
    variable debugSync
    variable debugMode
    if {$debugMode} {
	set id [$mainWindow.varsf.listbox curselection]
	set name [$mainWindow.varsf.listbox get $id]
	set debugSync "Debugger::showVarValue $name"
    }
}
proc Debugger::showVarValue var {
    variable mainWindow
    variable debugSync
    variable selectedLevel
    if {[uplevel #$selectedLevel "array exists ${var}"]} {
        set value [uplevel #$selectedLevel "::array get ${var}"]
    } else {
        set value [uplevel #$selectedLevel "::set ${var}"]
    }
    $mainWindow.vartext delete 1.0 end
    $mainWindow.vartext insert 1.0 $value
}
proc Debugger::endDebugging {} {
    variable mainWindow
    variable debugMode 0
    variable notHideDebugger
    if {$notHideDebugger} {
	$mainWindow.levelf.listbox delete 0 end
	$mainWindow.varsf.listbox delete 0 end
	$mainWindow.procTextFrame.text delete 1.0 end
	$mainWindow.buttons.resume configure -state disabled
	$mainWindow.buttons.stepInto configure -state disabled
	$mainWindow.buttons.stepOver configure -state disabled
    } else {
	wm withdraw $mainWindow
    }
}
proc Debugger::invokeResume {} {
    variable debugSync
    set debugSync {Debugger::endDebugging; break}
}
proc Debugger::invokeStepInto {} {
    variable debugSync
    set debugSync "atk::stepInto; break"
}
proc Debugger::invokeStepOver {} {
    variable debugSync
    set debugSync "atk::stepOver; break"
}
proc Debugger::displayInfo {} {
    variable mainWindow
    tk_messageBox -title "ATK Debugger" -message "ATKDebugger version 0.2 11.2003\n Author: Artur Trzewik\nGNU Public License Software\nATKDebugger comes with ABSOLUTELY NO WARRANTY See the GNU General Public License for more details\n\nhttp://www.xdobry.de/atkdebugger" -type ok -parent $mainWindow
}
proc Debugger::debuggFile {} {
    variable mainWindow
    variable debugMode
    if {$debugMode} return
    set file [tk_getOpenFile -parent $mainWindow]
    if {$file!=""} {
	atk::debugScript [list source $file]
    }
}
proc Debugger::startBreakpointsBrowser {} {
    variable breakpointsWindow
    if {$breakpointsWindow!="" && [winfo exists $breakpointsWindow]} {
	return
    }
    set breakpointsWindow [toplevel .breakpointsFrame]
    wm title $breakpointsWindow {Breakpoints Browser}

    set win [frame $breakpointsWindow.breakpoints]
    scrollbar $win.scroll -command [list $win.listbox yview] -takefocus 0
    listbox $win.listbox -yscroll [list $win.scroll set] -exportselection no -height 20 -width 50
    pack $win.scroll -side right -fill y
    pack $win.listbox -expand yes -fill both    

    set win [frame $breakpointsWindow.methods]
    scrollbar $win.scroll -command [list $win.listbox yview] -takefocus 0
    listbox $win.listbox -yscroll [list $win.scroll set]  -exportselection no -height 20 -width 50
    pack $win.scroll -side right -fill y
    pack $win.listbox -expand yes -fill both    


    button $breakpointsWindow.remove -text "Remove" -command Debugger::removeBreakpoint
    button $breakpointsWindow.add -text "Add" -command Debugger::addBreakpoint
    button $breakpointsWindow.refresh -text "Refresh" -command Debugger::refreshMethods

    grid $breakpointsWindow.breakpoints -column 0 -row 0 -sticky news
    grid $breakpointsWindow.add -column 1 -row 0 -sticky n
    grid $breakpointsWindow.methods -column 2 -row 0 -sticky news
    grid $breakpointsWindow.remove -column 0 -row 1
    grid $breakpointsWindow.refresh -column 2 -row 1
   
    grid rowconfigure $breakpointsWindow 0 -weight 1
    grid columnconfigure $breakpointsWindow 0 -weight 1
    grid columnconfigure $breakpointsWindow 2 -weight 1

    Debugger::refreshMethods
}
proc Debugger::refreshBreakpoints {} {
    variable breakpointsWindow
    variable bmethods [list]
    $breakpointsWindow.breakpoints.listbox delete 0 end
    foreach breakHandle [atk::breakpoint info] {
	lappend bmethods [lindex [atk::breakpoint info $breakHandle] 0]
    }
    eval $breakpointsWindow.breakpoints.listbox insert 0 [lsort $bmethods]
}
proc Debugger::getProcsFromNamespace {namesp} {
    variable bmethods
    set res [list]
    foreach p [namespace eval $namesp info procs] {
        set p [string trimleft [namespace eval $namesp [list namespace which -command $p]] :]
	if {[lsearch $bmethods $p]<0} {
	    lappend res $p
	}
    }
    foreach chns [namespace children $namesp] {
        set res [concat $res [Debugger::getProcsFromNamespace $chns]]
    }
    return $res
}
proc Debugger::refreshMethods {} {
    variable breakpointsWindow
    $breakpointsWindow.methods.listbox delete 0 end
    Debugger::refreshBreakpoints
    set methods [Debugger::getProcsFromNamespace ::]
    eval $breakpointsWindow.methods.listbox insert 0 [lsort -unique $methods]
}
proc Debugger::removeBreakpoint {} {
    variable breakpointsWindow
    set handles [atk::breakpoint info]
    foreach method [$breakpointsWindow.breakpoints.listbox curselection] {
	foreach b $handles {
            if {[lindex [atk::breakpoint info $b] 0]==[$breakpointsWindow.breakpoints.listbox get $method]} {
		atk::breakpoint remove $b
		break
	    }
     	}
    }
    Debugger::refreshMethods
}
proc Debugger::addBreakpoint {} {
    variable breakpointsWindow
    foreach method [$breakpointsWindow.methods.listbox curselection] {
	atk::breakpoint add [$breakpointsWindow.methods.listbox get $method]
    }
    Debugger::refreshMethods
}

atk::debugproc {
    global errorInfo 
    if {[catch {Debugger::atkDebugger}]} {
	puts $errorInfo
    }
}

# if arguments than start script to debug
if {[llength $argv]==1} {
    set script [lindex $argv 0]
    set argv [lrange $argv 1 end]
    atk::debugScript [list source $script]
} else {
    set Debugger::debugMode 0
    set Debugger::notHideDebugger 1
    Debugger::InitGUIThenNeeded
    Debugger::endDebugging
}
