#!/home/artur/programs/tcl8.4.3-atkdebugger/unix/tclsh

# I do not use package require atkdebugger to make passible
# to do the test with make test without the need to install atkdebugger in
# tcl target library fist.
# normall after installation you should use package require atkdebugger

if {[file exists libatkdebugger0.21.so]} {
    load libatkdebugger0.21.so
} else {
    package require atkdebugger
}

atk::debugproc

atk::debugproc debugproc

puts "debugproc introspection [atk::debugproc]"

puts "debuglevel without debug [atk::debuglevel]"

proc printExecutionPoint level {
    set res [atk::runplace [atk::debuglevel]]
    set source [lindex $res 0]
    set beginn [lindex $res 1]
    set len [lindex $res 2]
    set endexecution [expr {$beginn + $len}]
    set source "[string range $source 0 [expr {$beginn-1}]]>>>>[string range $source $beginn [expr {$endexecution-1}]]<<<<[string range $source $endexecution end]"
    puts $source
}

set debugCount 0

proc debugproc {} {
    global debugCount
    incr debugCount
    puts "Debugproc was invoked [atk::debuglevel]"
    puts "logpoint 1"
    catch {printExecutionPoint [atk::debuglevel]}
    global errorInfo
    puts $errorInfo
}

proc testproc {} {
    set a 12
    set b $a
    puts "step 1"
    atk::halt
    puts "step 2"
    set b $a
}

puts "entering testproc"
testproc

proc testproc2 {} {
    set a 2
    testproc
    set a
}

proc testproc3 {{p1 0}} {
    set a 2
    puts "testproc3"
    set a [expr {$a+1}]
    set b [list $a 3 4]
}
proc testproc4 {} {
    set a 3
}

testproc2

# Test der status setzen

atk::debugstatus 0
if {[atk::debugstatus]} {error "atk::debugstatus"}
testproc2

atk::debugstatus 1

# catch {atk::stepInto}
# puts $errorInfo

# catch {atk::stepOver}
# puts $errorInfo

# catch {atk::runplace 3}
# puts $errorInfo

# catch {atk::debuglevel}
# puts $errorInfo

# Testing breakpoints setting

puts "setting breakpoint"
set bh1 [atk::breakpoint add testproc3]
puts "handle for breakpoint 1 is $bh1"

puts "atk::breakpoints info all breakpoints [atk::breakpoint info]"

puts "info for breakpoint 1 [atk::breakpoint info $bh1]"

# invoking seting breakpoints

set dc $debugCount
puts "testing breakpoint $debugCount $dc"
testproc3
puts "testing breakpoint $debugCount $dc"
if {$debugCount==$dc} {
    error "breakpoint does not work."
}


if {![lindex [atk::breakpoint info $bh1] 2]} {
    error "breakpoint should be active"
}
atk::breakpoint configure $bh1 -active 0
if {[lindex [atk::breakpoint info $bh1] 2]} {
    error "breakpoint should not be active"
}
set dc $debugCount
testproc3
if {$debugCount!=$dc} {
    error "breakpoint should not be active."
}
atk::breakpoint configure $bh1 -active 1
if {![lindex [atk::breakpoint info $bh1] 2]} {
    error "breakpoint should be active"
}
# clean entering information

puts "test after activation"
set dc $debugCount
testproc3
if {$debugCount==$dc} {
    error "breakpoint should be active."
}

puts "Testing breakpoint expression"
atk::breakpoint configure $bh1 -expression {expr 0}
set dc $debugCount
testproc3
if {$debugCount!=$dc} {
    error "breakpoint expression prevent it"
}
atk::breakpoint configure $bh1 -expression {expr 1}
set dc $debugCount
testproc3
if {$debugCount==$dc} {
    error "breakpoint expression should return true"
}
# Set new breakpoint with expression
atk::breakpoint remove $bh1
set bh1 [atk::breakpoint add testproc3 0 {expr 0}]
set dc $debugCount
testproc3
if {$debugCount!=$dc} {
    error "breakpoint expression prevent it"
}
# Testing context dependent breakpoint
atk::breakpoint configure $bh1 -expression {expr $p1}
set dc $debugCount
testproc3 1
if {$debugCount==$dc} {
    error "breakpoint expression should return true"
}
set dc $debugCount
testproc3 0
if {$debugCount!=$dc} {
    error "breakpoint expression prevent it"
}

# Testing removing
atk::breakpoint remove $bh1
if {![catch {atk::breakpoint info $bh1}]} {
    error "should produce error
}

puts "end of test"
